# When connecting two faces it splits the starting top edge.
# It does not accept it when you put in only lines.
# It fails when there are more lonely points on one side than edges on the other.
require 'sketchup.rb'
def clean_selection(entities)
	trash = []
	Sketchup.active_model.start_operation "Clean Selection"
	selection = Sketchup.active_model.selection
	if (entities)
		selection.clear
		selection.add entities
	end
	selection.each {|looptemp|trash.push(looptemp) if (looptemp.typename == "Edge" && looptemp.faces.length == 0)}
	selection.each {|looptemp|trash.push(looptemp) if (looptemp.typename == "Edge" && looptemp.faces.length == 2 && looptemp.faces[0].material == looptemp.faces[1].material && looptemp.faces[0].normal.parallel?(looptemp.faces[1].normal))}
	trash.each {|looptemp|Sketchup.active_model.entities.erase_entities(looptemp)}
	Sketchup.active_model.commit_operation
	return
end
def skin
	entities = Sketchup.active_model.active_entities
	selection = Sketchup.active_model.selection
	allfaces = []
	baseedges = []
	basepoints = []
	ridgeedges = []
	ridgepoints = []
	lastpoints = []
	startstops = []
	highpoint = nil
	highnext = nil
	lowpoint = nil
	lownext = nil
	forepoint = nil
	low = nil
	endless = false
	selection.each {|looptemp| baseedges.push(looptemp)}
	allfaces = baseedges.reject{|looptemp| looptemp.typename != "Face"}
	if (allfaces.length == 0)
		UI.messagebox "I am sorry but you must select at least one face, a newer version than this (1.7) will probably let you connect lines."
		return false
	end
	baseedges.reject!{|looptemp| looptemp.typename != "Edge"}
	ridgeedges.push(baseedges.pop)
	ridgeedges.each do
		|looptemp|
		looptemp.vertices.each do
			|loopt|
			(loopt.edges & baseedges).each do
				|lpt|
				ridgeedges.push lpt
				baseedges.delete lpt
			end
		end
	end
	Sketchup.active_model.start_operation "Skin"
	((ridgeedges + baseedges).collect {|looptemp| looptemp.vertices}.flatten.uniq.reject {|looptemp| (looptemp.edges & (ridgeedges + baseedges)).length > 1}).each do
		|looptemp|
		if (ridgeedges.collect{|loopt| loopt.vertices}.flatten.uniq.include? looptemp)
			highpoint = looptemp
			lowpoint = (baseedges.collect {|loopt| loopt.vertices}.flatten.uniq.sort {|a,b|a.position.distance(highpoint) <=> b.position.distance(highpoint)})[0]
			forepoint = (lowpoint.edges.collect {|loopt| loopt.vertices}.flatten.uniq.sort {|a,b|a.position.distance(highpoint) <=> b.position.distance(highpoint)})[1]
			low = false
		else
			lowpoint = looptemp
			highpoint = (ridgeedges.collect {|loopt| loopt.vertices}.flatten.uniq.sort {|a,b|a.position.distance(lowpoint) <=> b.position.distance(lowpoint)})[0]
			forepoint = ((highpoint.edges & ridgeedges).collect {|loopt| loopt.vertices}.flatten.uniq.sort {|a,b|a.position.distance(lowpoint) <=> b.position.distance(lowpoint)})[1]
			low = true
		end
		allfaces.push(entities.add_face(highpoint, lowpoint, forepoint))
		if (low)
			startstops.push([lowpoint, highpoint], [lowpoint, forepoint])
		else
			startstops.push([lowpoint, highpoint], [forepoint, highpoint])
		end
	end
	if (startstops.length == 0)
		highpoint = Geom::Point3d.linear_combination(1, ridgeedges[0].vertices[0].position, 1, ridgeedges[0].vertices[1].position)
		lowpoint = baseedges.collect{|looptemp| looptemp.vertices}.flatten.uniq.sort{|a,b|a.position.distance(highpoint) <=> b.position.distance(highpoint)}[0]
		highpoint = ridgeedges[0].vertices.sort{|a,b|a.position.distance(lowpoint) <=> b.position.distance(lowpoint)}[0]
		startstops.push([lowpoint, highpoint])
		startstops.push([ridgeedges[0].vertices.reject{|looptemp|looptemp == highpoint}.shift])
		startstops[1].unshift(startstops[0][0].edges.collect{|looptemp|looptemp.vertices}.flatten.reject {|looptemp|looptemp == startstops[0][0]}.sort{|a,b|a.position.distance(startstops[1][0]) <=> b.position.distance(startstops[1][0])}[0])
		if (startstops[0][0].position.distance(startstops[1][1].position) < startstops[0][1].position.distance(startstops[1][0].position))
			entities.add_face(startstops[0][0], startstops[0][1], startstops[1][1])
			entities.add_face(startstops[1][1], startstops[1][0], startstops[0][0])
		else
			entities.add_face(startstops[0][0], startstops[0][1], startstops[1][0])
			entities.add_face(startstops[1][1], startstops[1][0], startstops[0][1])
		end
		lastpoints = [[startstops[1][0], startstops[1][1]]]
		endless = true
		forepoint = 2
	end
	lastpoints.push [lowpoint, highpoint]
	startstops.delete(lastpoints)
	while (forepoint != nil)
		forepoint = nil if (forepoint != 2)
		forepoint = 1 if (forepoint == 2)
		lownext = nil
		highnext = nil
		lownext = ((lowpoint.edges & baseedges).reject {|looptemp| (looptemp.faces & allfaces).length > 1}.collect {|looptemp| looptemp.vertices}.flatten.uniq.reject {|looptemp| looptemp == lowpoint || lastpoints.flatten.include?(looptemp)}.sort {|a,b|a.position.distance(highpoint) <=> b.position.distance(highpoint)})[0]
		highnext = ((highpoint.edges & ridgeedges).reject {|looptemp| (looptemp.faces & allfaces).length > 1}.collect {|looptemp| looptemp.vertices}.flatten.uniq.reject {|looptemp| looptemp == highpoint || lastpoints.flatten.include?(looptemp)}.sort {|a,b|a.position.distance(lowpoint) <=> b.position.distance(lowpoint)})[0]
		lownext = lowpoint if (lownext == nil)
		highnext = highpoint if (highnext == nil)
		if ((ridgeedges + baseedges).delete_if {|looptemp| (looptemp.faces & allfaces).length >= 2}.length == 0)
		elsif (forepoint != 1 && startstops.delete([lowpoint, highpoint]) != nil && endless == false)
			lowpoint = startstops[0][0]
			highpoint = startstops[0][1]
			startstops.shift
			forepoint = 1
			lastpoints = [[lowpoint, highpoint]]
		elsif ((lownext.position.distance(highpoint) < highnext.position.distance(lowpoint) && lownext != lowpoint) || (highpoint == highnext && lowpoint != lownext))
			forepoint = lownext
			low = true
		elsif ((lownext.position.distance(highpoint) >= highnext.position.distance(lowpoint) && highnext != highpoint) || (lowpoint == lownext && highnext != highpoint))
			forepoint = highnext
			low = false
		else
			UI.messagebox("Drat... if the code got here, then the operation is unsucsessful. Please check if your problem is described in the description (Windows/Preferences/Extensions/Skin).",MB_MULTILINE) 
			Sketchup.active_model.abort_operation
			return false
		end
		lastpoints.push [lowpoint, highpoint]
		allfaces.push(entities.add_face(highpoint, lowpoint, forepoint)) if (forepoint != 1 && forepoint != nil)
		if (low && forepoint != 1)
			lowpoint = forepoint
		elsif (forepoint != 1)
			highpoint = forepoint
		end
		if (endless && (lastpoints[1] & lastpoints.last).length == 0)
			lastpoints.shift
			endless = false
		end
	end
	temp = []
	allfaces.each {|looptemp| temp.push(looptemp.edges)}
	allfaces = temp + allfaces
	allfaces.flatten!
	allfaces.uniq!
	clean_selection allfaces
	Sketchup.active_model.commit_operation
	return true
end
if not (file_loaded? "skin.rb")
	add_separator_to_menu("Tools")
	dbtools = UI.menu("Tools").add_submenu("D.B.Tools")
		dbtools.add_item("Clean Selection") {clean_selection(nil)}
		dbtools.add_item("Skin") {skin}
end
skin = SketchupExtension.new "Skin", "skin.rb"
skin.description = "A tool that takes two selected sets of entities and connects them. Known bugs for this version (1.8) are: #1 When connecting two faces it splits a random edge. #2 It does not accept it when you put in only lines. #3 It fails when there are more lonely points (points with only one edge) on one side than edges on the other."
skin.version = "1.8"
skin.creator = "Darrel Belvin"
skin.copyright = "2007, Darrel Belvin"
Sketchup.register_extension skin
file_loaded("skin.rb")